@@ -1,6 +1,9 @@
+# frozen_string_literal: true
+
 class Contract < ActiveRecord::Base
   belongs_to :company
-  belongs_to :developer
+  belongs_to :developer, primary_key: :id
+  belongs_to :firm, foreign_key: "company_id"
 
   before_save :hi
   after_save :bye
@@ -17,3 +20,7 @@ class Contract < ActiveRecord::Base
     @bye_count += 1
   end
 end
+
+class NewContract < Contract
+  validates :company_id, presence: true
+end
